@@ -477,15 +477,21 @@ bool SymlinkingPrebuiltDictionaries::Run(Deployer* deployer) {
     fs::path entry(test->path());
     if (fs::is_symlink(entry)) {
       try {
-        auto target_path = fs::canonical(entry);
-        if (target_path.has_parent_path() &&
-            fs::equivalent(shared_data_path, target_path.parent_path())) {
+        // a symlink becomes dangling if the target file is no longer provided
+        boost::system::error_code ec;
+        auto target_path = fs::canonical(entry, ec);
+        bool bad_link = bool(ec);
+        bool linked_to_shared_data =
+            !bad_link &&
+            target_path.has_parent_path() &&
+            fs::equivalent(shared_data_path, target_path.parent_path());
+        if (bad_link || linked_to_shared_data) {
           LOG(INFO) << "removing symlink: " << entry.filename().string();
           fs::remove(entry);
         }
       }
       catch (const fs::filesystem_error& ex) {
-        LOG(ERROR) << ex.what();
+        LOG(ERROR) << entry << ": " << ex.what();
         success = false;
       }
     }
