@@ -1,15 +1,15 @@
 //
-// Copyleft RIME Developers
-// License: GPLv3
+// Copyright RIME Developers
+// Distributed under the BSD License
 //
 // 2011-08-09 GONG Chen <chen.sst@gmail.com>
 //
 #include <cstring>
-#include <functional>
 #include <sstream>
 #include <boost/format.hpp>
 #include <rime/common.h>
 #include <rime/composition.h>
+#include <rime/config.h>
 #include <rime/context.h>
 #include <rime/deployer.h>
 #include <rime/key_event.h>
@@ -22,6 +22,7 @@
 #include <rime/signature.h>
 #include <rime_api.h>
 
+using namespace rime;
 using namespace std::placeholders;
 
 // assuming member is a pointer in struct *p
@@ -29,7 +30,7 @@ using namespace std::placeholders;
 
 static void setup_deployer(RimeTraits *traits) {
   if (!traits) return;
-  rime::Deployer &deployer(rime::Service::instance().deployer());
+  Deployer &deployer(Service::instance().deployer());
   if (PROVIDED(traits, shared_data_dir))
     deployer.shared_data_dir = traits->shared_data_dir;
   if (PROVIDED(traits, user_data_dir))
@@ -43,7 +44,7 @@ static void setup_deployer(RimeTraits *traits) {
 }
 
 RIME_API void RimeSetupLogging(const char* app_name) {
-  rime::SetupLogging(app_name);
+  SetupLogging(app_name);
 }
 
 #if RIME_BUILD_SHARED_LIBS
@@ -67,47 +68,46 @@ RIME_API void RimeSetup(RimeTraits *traits) {
 
   setup_deployer(traits);
   if (PROVIDED(traits, app_name)) {
-    rime::SetupLogging(traits->app_name);
+    SetupLogging(traits->app_name);
   }
 }
 
 RIME_API void RimeSetNotificationHandler(RimeNotificationHandler handler,
                                          void* context_object) {
   if (handler) {
-    rime::Service::instance().SetNotificationHandler(
+    Service::instance().SetNotificationHandler(
         std::bind(handler, context_object, _1, _2, _3));
   }
   else {
-    rime::Service::instance().ClearNotificationHandler();
+    Service::instance().ClearNotificationHandler();
   }
 }
 
 RIME_API void RimeInitialize(RimeTraits *traits) {
   setup_deployer(traits);
-  rime::LoadModules(
-      PROVIDED(traits, modules) ? traits->modules : rime::kDefaultModules);
-  rime::Service::instance().StartService();
+  LoadModules(PROVIDED(traits, modules) ? traits->modules : kDefaultModules);
+  Service::instance().StartService();
 }
 
 RIME_API void RimeFinalize() {
   RimeJoinMaintenanceThread();
-  rime::Service::instance().StopService();
-  rime::Registry::instance().Clear();
-  rime::ModuleManager::instance().UnloadModules();
+  Service::instance().StopService();
+  Registry::instance().Clear();
+  ModuleManager::instance().UnloadModules();
 }
 
 RIME_API Bool RimeStartMaintenance(Bool full_check) {
-  rime::LoadModules(rime::kDeployerModules);
-  rime::Deployer &deployer(rime::Service::instance().deployer());
+  LoadModules(kDeployerModules);
+  Deployer &deployer(Service::instance().deployer());
   deployer.RunTask("clean_old_log_files");
   if (!deployer.RunTask("installation_update")) {
     return False;
   }
   if (!full_check) {
-    rime::TaskInitializer args(
-        std::make_pair<std::string, std::string>(
-            "default.yaml", "config_version"));
-    if (!deployer.RunTask("config_file_update", args)) {
+    TaskInitializer args{
+      vector<string>{deployer.user_data_dir, deployer.shared_data_dir}
+    };
+    if (!deployer.RunTask("detect_modifications", args)) {
       return False;
     }
     LOG(INFO) << "changes detected; starting maintenance.";
@@ -124,12 +124,12 @@ RIME_API Bool RimeStartMaintenanceOnWorkspaceChange() {
 }
 
 RIME_API Bool RimeIsMaintenancing() {
-  rime::Deployer &deployer(rime::Service::instance().deployer());
+  Deployer &deployer(Service::instance().deployer());
   return Bool(deployer.IsMaintenanceMode());
 }
 
 RIME_API void RimeJoinMaintenanceThread() {
-  rime::Deployer &deployer(rime::Service::instance().deployer());
+  Deployer &deployer(Service::instance().deployer());
   deployer.JoinMaintenanceThread();
 }
 
@@ -137,17 +137,16 @@ RIME_API void RimeJoinMaintenanceThread() {
 
 RIME_API void RimeDeployerInitialize(RimeTraits *traits) {
   setup_deployer(traits);
-  rime::LoadModules(
-      PROVIDED(traits, modules) ? traits->modules : rime::kDeployerModules);
+  LoadModules(PROVIDED(traits, modules) ? traits->modules : kDeployerModules);
 }
 
 RIME_API Bool RimePrebuildAllSchemas() {
-  rime::Deployer &deployer(rime::Service::instance().deployer());
+  Deployer &deployer(Service::instance().deployer());
   return Bool(deployer.RunTask("prebuild_all_schemas"));
 }
 
 RIME_API Bool RimeDeployWorkspace() {
-  rime::Deployer &deployer(rime::Service::instance().deployer());
+  Deployer &deployer(Service::instance().deployer());
   return Bool(deployer.RunTask("installation_update") &&
               deployer.RunTask("workspace_update") &&
               deployer.RunTask("user_dict_upgrade") &&
@@ -155,22 +154,20 @@ RIME_API Bool RimeDeployWorkspace() {
 }
 
 RIME_API Bool RimeDeploySchema(const char *schema_file) {
-  rime::Deployer &deployer(rime::Service::instance().deployer());
-  return Bool(deployer.RunTask("schema_update", std::string(schema_file)));
+  Deployer &deployer(Service::instance().deployer());
+  return Bool(deployer.RunTask("schema_update", string(schema_file)));
 }
 
 RIME_API Bool RimeDeployConfigFile(const char *file_name,
                                    const char *version_key) {
-  rime::Deployer& deployer(rime::Service::instance().deployer());
-  rime::TaskInitializer args(
-      std::make_pair<std::string, std::string>(
-          file_name, version_key));
+  Deployer& deployer(Service::instance().deployer());
+  TaskInitializer args(make_pair<string, string>(file_name, version_key));
   return Bool(deployer.RunTask("config_file_update", args));
 }
 
 RIME_API Bool RimeSyncUserData() {
   RimeCleanupAllSessions();
-  rime::Deployer& deployer(rime::Service::instance().deployer());
+  Deployer& deployer(Service::instance().deployer());
   deployer.ScheduleTask("installation_update");
   deployer.ScheduleTask("backup_config_files");
   deployer.ScheduleTask("user_dict_sync");
@@ -180,43 +177,43 @@ RIME_API Bool RimeSyncUserData() {
 // session management
 
 RIME_API RimeSessionId RimeCreateSession() {
-  return rime::Service::instance().CreateSession();
+  return Service::instance().CreateSession();
 }
 
 RIME_API Bool RimeFindSession(RimeSessionId session_id) {
-  return Bool(session_id && rime::Service::instance().GetSession(session_id));
+  return Bool(session_id && Service::instance().GetSession(session_id));
 }
 
 RIME_API Bool RimeDestroySession(RimeSessionId session_id) {
-  return Bool(rime::Service::instance().DestroySession(session_id));
+  return Bool(Service::instance().DestroySession(session_id));
 }
 
 RIME_API void RimeCleanupStaleSessions() {
-  rime::Service::instance().CleanupStaleSessions();
+  Service::instance().CleanupStaleSessions();
 }
 
 RIME_API void RimeCleanupAllSessions() {
-  rime::Service::instance().CleanupAllSessions();
+  Service::instance().CleanupAllSessions();
 }
 
 // input
 
 RIME_API Bool RimeProcessKey(RimeSessionId session_id, int keycode, int mask) {
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session)
     return False;
-  return Bool(session->ProcessKey(rime::KeyEvent(keycode, mask)));
+  return Bool(session->ProcessKey(KeyEvent(keycode, mask)));
 }
 
 RIME_API Bool RimeCommitComposition(RimeSessionId session_id) {
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session)
     return False;
   return Bool(session->CommitComposition());
 }
 
 RIME_API void RimeClearComposition(RimeSessionId session_id) {
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session)
     return;
   session->ClearComposition();
@@ -224,19 +221,32 @@ RIME_API void RimeClearComposition(RimeSessionId session_id) {
 
 // output
 
+static void rime_candidate_copy(RimeCandidate* dest, const an<Candidate>& src) {
+  dest->text = new char[src->text().length() + 1];
+  std::strcpy(dest->text, src->text().c_str());
+  string comment(src->comment());
+  if (!comment.empty()) {
+    dest->comment = new char[comment.length() + 1];
+    std::strcpy(dest->comment, comment.c_str());
+  }
+  else {
+    dest->comment = nullptr;
+  }
+  dest->reserved = nullptr;
+}
+
 RIME_API Bool RimeGetContext(RimeSessionId session_id, RimeContext* context) {
   if (!context || context->data_size <= 0)
     return False;
   RIME_STRUCT_CLEAR(*context);
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session)
     return False;
-  rime::Context *ctx = session->context();
+  Context *ctx = session->context();
   if (!ctx)
     return False;
   if (ctx->IsComposing()) {
-    rime::Preedit preedit;
-    ctx->GetPreedit(&preedit, ctx->get_option("soft_cursor"));
+    Preedit preedit = ctx->GetPreedit();
     context->composition.length = preedit.text.length();
     context->composition.preedit = new char[preedit.text.length() + 1];
     std::strcpy(context->composition.preedit, preedit.text.c_str());
@@ -244,7 +254,7 @@ RIME_API Bool RimeGetContext(RimeSessionId session_id, RimeContext* context) {
     context->composition.sel_start = preedit.sel_start;
     context->composition.sel_end = preedit.sel_end;
     if (RIME_STRUCT_HAS_MEMBER(*context, context->commit_text_preview)) {
-      std::string commit_text(ctx->GetCommitText());
+      string commit_text(ctx->GetCommitText());
       if (!commit_text.empty()) {
         context->commit_text_preview = new char[commit_text.length() + 1];
         std::strcpy(context->commit_text_preview, commit_text.c_str());
@@ -252,14 +262,14 @@ RIME_API Bool RimeGetContext(RimeSessionId session_id, RimeContext* context) {
     }
   }
   if (ctx->HasMenu()) {
-    rime::Segment &seg(ctx->composition()->back());
+    Segment &seg(ctx->composition().back());
     int page_size = 5;
-    rime::Schema *schema = session->schema();
+    Schema *schema = session->schema();
     if (schema)
       page_size = schema->page_size();
     int selected_index = seg.selected_index;
     int page_no = selected_index / page_size;
-    rime::unique_ptr<rime::Page> page(seg.menu->CreatePage(page_size, page_no));
+    the<Page> page(seg.menu->CreatePage(page_size, page_no));
     if (page) {
       context->menu.page_size = page_size;
       context->menu.page_no = page_no;
@@ -268,25 +278,27 @@ RIME_API Bool RimeGetContext(RimeSessionId session_id, RimeContext* context) {
       int i = 0;
       context->menu.num_candidates = page->candidates.size();
       context->menu.candidates = new RimeCandidate[page->candidates.size()];
-      for (const rime::shared_ptr<rime::Candidate> &cand : page->candidates) {
+      for (const an<Candidate> &cand : page->candidates) {
         RimeCandidate* dest = &context->menu.candidates[i++];
-        dest->text = new char[cand->text().length() + 1];
-        std::strcpy(dest->text, cand->text().c_str());
-        std::string comment(cand->comment());
-        if (!comment.empty()) {
-          dest->comment = new char[comment.length() + 1];
-          std::strcpy(dest->comment, comment.c_str());
-        }
-        else {
-          dest->comment = NULL;
-        }
+        rime_candidate_copy(dest, cand);
       }
       if (schema) {
-        const std::string& select_keys(schema->select_keys());
+        const string& select_keys(schema->select_keys());
         if (!select_keys.empty()) {
           context->menu.select_keys = new char[select_keys.length() + 1];
           std::strcpy(context->menu.select_keys, select_keys.c_str());
         }
+        Config* config = schema->config();
+        an<ConfigList>  select_labels = config->GetList("menu/alternative_select_labels");
+        if (select_labels && (size_t)page_size <= select_labels->size()) {
+          context->select_labels = new char*[page_size];
+          for (size_t i = 0; i < (size_t)page_size; ++i) {
+            an<ConfigValue> value = select_labels->GetValueAt(i);
+            string label = value->str();
+            context->select_labels[i] = new char[label.length() + 1];
+            std::strcpy(context->select_labels[i], label.c_str());
+          }
+        }
       }
     }
   }
@@ -303,6 +315,12 @@ RIME_API Bool RimeFreeContext(RimeContext* context) {
   }
   delete[] context->menu.candidates;
   delete[] context->menu.select_keys;
+  if (RIME_STRUCT_HAS_MEMBER(*context, context->select_labels) && context->select_labels) {
+    for (int i = 0; i < context->menu.page_size; ++i) {
+      delete[] context->select_labels[i];
+    }
+    delete[] context->select_labels;
+  }
   if (RIME_STRUCT_HAS_MEMBER(*context, context->commit_text_preview)) {
     delete[] context->commit_text_preview;
   }
@@ -314,10 +332,10 @@ RIME_API Bool RimeGetCommit(RimeSessionId session_id, RimeCommit* commit) {
   if (!commit)
     return False;
   RIME_STRUCT_CLEAR(*commit);
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session)
     return False;
-  const std::string& commit_text(session->commit_text());
+  const string& commit_text(session->commit_text());
   if (!commit_text.empty()) {
     commit->text = new char[commit_text.length() + 1];
     std::strcpy(commit->text, commit_text.c_str());
@@ -339,18 +357,18 @@ RIME_API Bool RimeGetStatus(RimeSessionId session_id, RimeStatus* status) {
   if (!status || status->data_size <= 0)
     return False;
   RIME_STRUCT_CLEAR(*status);
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session)
     return False;
-  rime::Schema *schema = session->schema();
-  rime::Context *ctx = session->context();
+  Schema *schema = session->schema();
+  Context *ctx = session->context();
   if (!schema || !ctx)
     return False;
   status->schema_id = new char[schema->schema_id().length() + 1];
   std::strcpy(status->schema_id, schema->schema_id().c_str());
   status->schema_name = new char[schema->schema_name().length() + 1];
   std::strcpy(status->schema_name, schema->schema_name().c_str());
-  status->is_disabled = rime::Service::instance().disabled();
+  status->is_disabled = Service::instance().disabled();
   status->is_composing = Bool(ctx->IsComposing());
   status->is_ascii_mode = Bool(ctx->get_option("ascii_mode"));
   status->is_full_shape = Bool(ctx->get_option("full_shape"));
@@ -369,33 +387,75 @@ RIME_API Bool RimeFreeStatus(RimeStatus* status) {
   return True;
 }
 
+// Accessing candidate list
+
+RIME_API Bool RimeCandidateListBegin(RimeSessionId session_id,
+                                     RimeCandidateListIterator* iterator) {
+  if (!iterator)
+    return False;
+  an<Session> session(Service::instance().GetSession(session_id));
+  if (!session)
+    return False;
+  Context *ctx = session->context();
+  if (!ctx || !ctx->HasMenu())
+    return False;
+  memset(iterator, 0, sizeof(RimeCandidateListIterator));
+  iterator->ptr = ctx->composition().back().menu.get();
+  iterator->index = -1;
+  return True;
+}
+
+RIME_API Bool RimeCandidateListNext(RimeCandidateListIterator* iterator) {
+  if (!iterator)
+    return False;
+  Menu *menu = reinterpret_cast<Menu*>(iterator->ptr);
+  if (!menu)
+    return False;
+  ++iterator->index;
+  if (auto cand = menu->GetCandidateAt((size_t)iterator->index)) {
+    delete[] iterator->candidate.text;
+    delete[] iterator->candidate.comment;
+    rime_candidate_copy(&iterator->candidate, cand);
+    return True;
+  }
+  return False;
+}
+
+RIME_API void RimeCandidateListEnd(RimeCandidateListIterator* iterator) {
+  if (!iterator)
+    return;
+  delete[] iterator->candidate.text;
+  delete[] iterator->candidate.comment;
+  memset(iterator, 0, sizeof(RimeCandidateListIterator));
+}
+
 // runtime options
 
 RIME_API void RimeSetOption(RimeSessionId session_id, const char* option, Bool value) {
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session)
     return;
-  rime::Context *ctx = session->context();
+  Context *ctx = session->context();
   if (!ctx)
     return;
   ctx->set_option(option, !!value);
 }
 
 RIME_API Bool RimeGetOption(RimeSessionId session_id, const char* option) {
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session)
     return False;
-  rime::Context *ctx = session->context();
+  Context *ctx = session->context();
   if (!ctx)
     return False;
   return Bool(ctx->get_option(option));
 }
 
 RIME_API void RimeSetProperty(RimeSessionId session_id, const char* prop, const char* value) {
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session)
     return;
-  rime::Context *ctx = session->context();
+  Context *ctx = session->context();
   if (!ctx)
     return;
   ctx->set_property(prop, value);
@@ -403,13 +463,13 @@ RIME_API void RimeSetProperty(RimeSessionId session_id, const char* prop, const
 
 RIME_API Bool RimeGetProperty(RimeSessionId session_id, const char* prop,
                               char* value, size_t buffer_size) {
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session)
     return False;
-  rime::Context *ctx = session->context();
+  Context *ctx = session->context();
   if (!ctx)
     return False;
-  std::string str_value(ctx->get_property(prop));
+  string str_value(ctx->get_property(prop));
   if (str_value.empty())
     return False;
   strncpy(value, str_value.c_str(), buffer_size);
@@ -420,23 +480,23 @@ RIME_API Bool RimeGetSchemaList(RimeSchemaList* output) {
   if (!output) return False;
   output->size = 0;
   output->list = NULL;
-  rime::Schema default_schema;
-  rime::Config* config = default_schema.config();
+  Schema default_schema;
+  Config* config = default_schema.config();
   if (!config) return False;
-  rime::ConfigListPtr schema_list = config->GetList("schema_list");
+  an<ConfigList> schema_list = config->GetList("schema_list");
   if (!schema_list || schema_list->size() == 0)
     return False;
   output->list = new RimeSchemaListItem[schema_list->size()];
   for (size_t i = 0; i < schema_list->size(); ++i) {
-    rime::ConfigMapPtr item = rime::As<rime::ConfigMap>(schema_list->GetAt(i));
+    an<ConfigMap> item = As<ConfigMap>(schema_list->GetAt(i));
     if (!item) continue;
-    rime::ConfigValuePtr schema_property = item->GetValue("schema");
+    an<ConfigValue> schema_property = item->GetValue("schema");
     if (!schema_property) continue;
-    const std::string &schema_id(schema_property->str());
+    const string &schema_id(schema_property->str());
     RimeSchemaListItem& x(output->list[output->size]);
     x.schema_id = new char[schema_id.length() + 1];
     strcpy(x.schema_id, schema_id.c_str());
-    rime::Schema schema(schema_id);
+    Schema schema(schema_id);
     x.name = new char[schema.schema_name().length() + 1];
     strcpy(x.name, schema.schema_name().c_str());
     x.reserved = NULL;
@@ -464,9 +524,9 @@ RIME_API void RimeFreeSchemaList(RimeSchemaList* schema_list) {
 }
 
 RIME_API Bool RimeGetCurrentSchema(RimeSessionId session_id, char* schema_id, size_t buffer_size) {
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session) return False;
-  rime::Schema* schema = session->schema();
+  Schema* schema = session->schema();
   if (!schema) return False;
   strncpy(schema_id, schema->schema_id().c_str(), buffer_size);
   return True;
@@ -474,37 +534,41 @@ RIME_API Bool RimeGetCurrentSchema(RimeSessionId session_id, char* schema_id, si
 
 RIME_API Bool RimeSelectSchema(RimeSessionId session_id, const char* schema_id) {
   if (!schema_id) return False;
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session) return False;
-  session->ApplySchema(new rime::Schema(schema_id));
+  session->ApplySchema(new Schema(schema_id));
   return True;
 }
 
 // config
 
-RIME_API Bool RimeSchemaOpen(const char *schema_id, RimeConfig* config) {
-  if (!schema_id || !config) return False;
-  rime::Config::Component* cc = rime::Config::Require("schema_config");
+static Bool open_config_in_component(const char* config_component,
+                                     const char* config_id,
+                                     RimeConfig* config) {
+  if (!config_id || !config) return False;
+  Config::Component* cc = Config::Require(config_component);
   if (!cc) return False;
-  rime::Config* c = cc->Create(schema_id);
+  Config* c = cc->Create(config_id);
   if (!c) return False;
   config->ptr = (void*)c;
   return True;
 }
 
+RIME_API Bool RimeSchemaOpen(const char *schema_id, RimeConfig* config) {
+  return open_config_in_component("schema", schema_id, config);
+}
+
 RIME_API Bool RimeConfigOpen(const char *config_id, RimeConfig* config) {
-  if (!config_id || !config) return False;
-  rime::Config::Component* cc = rime::Config::Require("config");
-  if (!cc) return False;
-  rime::Config* c = cc->Create(config_id);
-  if (!c) return False;
-  config->ptr = (void*)c;
-  return True;
+  return open_config_in_component("config", config_id, config);
+}
+
+RIME_API Bool RimeUserConfigOpen(const char* config_id, RimeConfig* config) {
+  return open_config_in_component("user_config", config_id, config);
 }
 
 RIME_API Bool RimeConfigClose(RimeConfig *config) {
   if (!config || !config->ptr) return False;
-  rime::Config *c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config *c = reinterpret_cast<Config*>(config->ptr);
   delete c;
   config->ptr = NULL;
   return True;
@@ -512,7 +576,7 @@ RIME_API Bool RimeConfigClose(RimeConfig *config) {
 
 RIME_API Bool RimeConfigGetBool(RimeConfig *config, const char *key, Bool *value) {
   if (!config || !key || !value) return False;
-  rime::Config *c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config *c = reinterpret_cast<Config*>(config->ptr);
   bool bool_value = false;
   if (c->GetBool(key, &bool_value)) {
     *value = Bool(bool_value);
@@ -523,22 +587,22 @@ RIME_API Bool RimeConfigGetBool(RimeConfig *config, const char *key, Bool *value
 
 RIME_API Bool RimeConfigGetInt(RimeConfig *config, const char *key, int *value) {
   if (!config || !key || !value) return False;
-  rime::Config *c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config *c = reinterpret_cast<Config*>(config->ptr);
   return Bool(c->GetInt(key, value));
 }
 
 RIME_API Bool RimeConfigGetDouble(RimeConfig *config, const char *key, double *value) {
   if (!config || !key || !value) return False;
-  rime::Config *c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config *c = reinterpret_cast<Config*>(config->ptr);
   return Bool(c->GetDouble(key, value));
 }
 
 RIME_API Bool RimeConfigGetString(RimeConfig *config, const char *key,
                                   char *value, size_t buffer_size) {
   if (!config || !key || !value) return False;
-  rime::Config *c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config *c = reinterpret_cast<Config*>(config->ptr);
   if (!c) return False;
-  std::string str_value;
+  string str_value;
   if (c->GetString(key, &str_value)) {
     std::strncpy(value, str_value.c_str(), buffer_size);
     return True;
@@ -548,9 +612,9 @@ RIME_API Bool RimeConfigGetString(RimeConfig *config, const char *key,
 
 RIME_API const char* RimeConfigGetCString(RimeConfig *config, const char *key) {
   if (!config || !key) return NULL;
-  rime::Config *c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config *c = reinterpret_cast<Config*>(config->ptr);
   if (!c) return NULL;
-  if (rime::ConfigValuePtr v = c->GetValue(key)) {
+  if (an<ConfigValue> v = c->GetValue(key)) {
     return v->str().c_str();
   }
   return NULL;
@@ -558,9 +622,9 @@ RIME_API const char* RimeConfigGetCString(RimeConfig *config, const char *key) {
 
 RIME_API Bool RimeConfigUpdateSignature(RimeConfig *config, const char* signer) {
   if (!config || !signer) return False;
-  rime::Config *c = reinterpret_cast<rime::Config*>(config->ptr);
-  rime::Deployer &deployer(rime::Service::instance().deployer());
-  rime::Signature sig(signer);
+  Config *c = reinterpret_cast<Config*>(config->ptr);
+  Deployer &deployer(Service::instance().deployer());
+  Signature sig(signer);
   return Bool(sig.Sign(c, &deployer));
 }
 
@@ -568,10 +632,10 @@ template <class T>
 struct RimeConfigIteratorImpl {
   typename T::Iterator iter;
   typename T::Iterator end;
-  std::string prefix;
-  std::string key;
-  std::string path;
-  RimeConfigIteratorImpl<T>(T& container, const std::string& root_path)
+  string prefix;
+  string key;
+  string path;
+  RimeConfigIteratorImpl<T>(T& container, const string& root_path)
       : iter(container.begin()),
         end(container.end()) {
     if (root_path.empty() || root_path == "/") {
@@ -592,13 +656,13 @@ RIME_API Bool RimeConfigBeginList(RimeConfigIterator* iterator,
   iterator->index = -1;
   iterator->key = NULL;
   iterator->path = NULL;
-  rime::Config *c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config *c = reinterpret_cast<Config*>(config->ptr);
   if (!c)
     return False;
-  rime::ConfigListPtr list = c->GetList(key);
+  an<ConfigList> list = c->GetList(key);
   if (!list)
     return False;
-  iterator->list = new RimeConfigIteratorImpl<rime::ConfigList>(*list, key);
+  iterator->list = new RimeConfigIteratorImpl<ConfigList>(*list, key);
   return True;
 }
 
@@ -611,11 +675,11 @@ RIME_API Bool RimeConfigBeginMap(RimeConfigIterator* iterator,
   iterator->index = -1;
   iterator->key = NULL;
   iterator->path = NULL;
-  rime::Config *c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config *c = reinterpret_cast<Config*>(config->ptr);
   if (!c) return False;
-  rime::ConfigMapPtr m = c->GetMap(key);
+  an<ConfigMap> m = c->GetMap(key);
   if (!m) return False;
-  iterator->map = new RimeConfigIteratorImpl<rime::ConfigMap>(*m, key);
+  iterator->map = new RimeConfigIteratorImpl<ConfigMap>(*m, key);
   return True;
 }
 
@@ -623,8 +687,8 @@ RIME_API Bool RimeConfigNext(RimeConfigIterator* iterator) {
   if (!iterator->list && !iterator->map)
     return False;
   if (iterator->list) {
-    RimeConfigIteratorImpl<rime::ConfigList>* p =
-        reinterpret_cast<RimeConfigIteratorImpl<rime::ConfigList>*>(iterator->list);
+    RimeConfigIteratorImpl<ConfigList>* p =
+        reinterpret_cast<RimeConfigIteratorImpl<ConfigList>*>(iterator->list);
     if (!p) return False;
     if (++iterator->index > 0)
       ++p->iter;
@@ -637,8 +701,8 @@ RIME_API Bool RimeConfigNext(RimeConfigIterator* iterator) {
     return True;
   }
   if (iterator->map) {
-    RimeConfigIteratorImpl<rime::ConfigMap>* p =
-        reinterpret_cast<RimeConfigIteratorImpl<rime::ConfigMap>*>(iterator->map);
+    RimeConfigIteratorImpl<ConfigMap>* p =
+        reinterpret_cast<RimeConfigIteratorImpl<ConfigMap>*>(iterator->map);
     if (!p) return False;
     if (++iterator->index > 0)
       ++p->iter;
@@ -656,24 +720,24 @@ RIME_API Bool RimeConfigNext(RimeConfigIterator* iterator) {
 RIME_API void RimeConfigEnd(RimeConfigIterator* iterator) {
   if (!iterator) return;
   if (iterator->list)
-    delete reinterpret_cast<RimeConfigIteratorImpl<rime::ConfigList>*>(iterator->list);
+    delete reinterpret_cast<RimeConfigIteratorImpl<ConfigList>*>(iterator->list);
   if (iterator->map)
-    delete reinterpret_cast<RimeConfigIteratorImpl<rime::ConfigMap>*>(iterator->map);
+    delete reinterpret_cast<RimeConfigIteratorImpl<ConfigMap>*>(iterator->map);
   memset(iterator, 0, sizeof(RimeConfigIterator));
 }
 
 
 RIME_API Bool RimeSimulateKeySequence(RimeSessionId session_id, const char *key_sequence) {
   LOG(INFO) << "simulate key sequence: " << key_sequence;
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session)
     return False;
-  rime::KeySequence keys;
+  KeySequence keys;
   if (!keys.Parse(key_sequence)) {
     LOG(ERROR) << "error parsing input: '" << key_sequence << "'";
     return False;
   }
-  for (const rime::KeyEvent& key : keys) {
+  for (const KeyEvent& key : keys) {
     session->ProcessKey(key);
   }
   return True;
@@ -682,50 +746,50 @@ RIME_API Bool RimeSimulateKeySequence(RimeSessionId session_id, const char *key_
 RIME_API Bool RimeRegisterModule(RimeModule* module) {
   if (!module || !module->module_name)
     return False;
-  rime::ModuleManager::instance().Register(module->module_name, module);
+  ModuleManager::instance().Register(module->module_name, module);
   return True;
 }
 
 RIME_API RimeModule* RimeFindModule(const char* module_name) {
-  return rime::ModuleManager::instance().Find(module_name);
+  return ModuleManager::instance().Find(module_name);
 }
 
 RIME_API Bool RimeRunTask(const char* task_name) {
   if (!task_name)
     return False;
-  rime::Deployer &deployer(rime::Service::instance().deployer());
+  Deployer &deployer(Service::instance().deployer());
   return Bool(deployer.RunTask(task_name));
 }
 
 RIME_API const char* RimeGetSharedDataDir() {
-  rime::Deployer &deployer(rime::Service::instance().deployer());
+  Deployer &deployer(Service::instance().deployer());
   return deployer.shared_data_dir.c_str();
 }
 
 RIME_API const char* RimeGetUserDataDir() {
-  rime::Deployer &deployer(rime::Service::instance().deployer());
+  Deployer &deployer(Service::instance().deployer());
   return deployer.user_data_dir.c_str();
 }
 
 RIME_API const char* RimeGetSyncDir() {
-  rime::Deployer &deployer(rime::Service::instance().deployer());
+  Deployer &deployer(Service::instance().deployer());
   return deployer.sync_dir.c_str();
 }
 
 RIME_API const char* RimeGetUserId() {
-  rime::Deployer &deployer(rime::Service::instance().deployer());
+  Deployer &deployer(Service::instance().deployer());
   return deployer.user_id.c_str();
 }
 
 RIME_API void RimeGetUserDataSyncDir(char* dir, size_t buffer_size) {
-  rime::Deployer &deployer(rime::Service::instance().deployer());
+  Deployer &deployer(Service::instance().deployer());
   strncpy(dir, deployer.user_data_sync_dir().c_str(), buffer_size);
 }
 
 RIME_API Bool RimeConfigInit(RimeConfig* config) {
   if (!config || config->ptr)
     return False;
-  config->ptr = (void*)new rime::Config;
+  config->ptr = (void*)new Config;
   return True;
 }
 
@@ -736,7 +800,7 @@ RIME_API Bool RimeConfigLoadString(RimeConfig* config, const char* yaml) {
   if (!config->ptr) {
     RimeConfigInit(config);
   }
-  rime::Config* c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config* c = reinterpret_cast<Config*>(config->ptr);
   std::istringstream iss(yaml);
   return Bool(c->LoadFromStream(iss));
 }
@@ -744,13 +808,13 @@ RIME_API Bool RimeConfigLoadString(RimeConfig* config, const char* yaml) {
 RIME_API Bool RimeConfigGetItem(RimeConfig* config, const char* key, RimeConfig* value) {
   if (!config || !key || !value)
     return False;
-  rime::Config* c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config* c = reinterpret_cast<Config*>(config->ptr);
   if (!c)
     return False;
   if (!value->ptr) {
     RimeConfigInit(value);
   }
-  rime::Config* v = reinterpret_cast<rime::Config*>(value->ptr);
+  Config* v = reinterpret_cast<Config*>(value->ptr);
   *v = c->GetItem(key);
   return True;
 }
@@ -758,12 +822,12 @@ RIME_API Bool RimeConfigGetItem(RimeConfig* config, const char* key, RimeConfig*
 RIME_API Bool RimeConfigSetItem(RimeConfig* config, const char* key, RimeConfig* value) {
   if (!config || !key)
     return False;
-  rime::Config* c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config* c = reinterpret_cast<Config*>(config->ptr);
   if (!c)
     return False;
-  rime::ConfigItemPtr item;
+  an<ConfigItem> item;
   if (value) {
-    if (rime::Config* v = reinterpret_cast<rime::Config*>(value->ptr)) {
+    if (Config* v = reinterpret_cast<Config*>(value->ptr)) {
       item = v->GetItem("");
     }
   }
@@ -773,16 +837,16 @@ RIME_API Bool RimeConfigSetItem(RimeConfig* config, const char* key, RimeConfig*
 RIME_API Bool RimeConfigSetBool(RimeConfig* config, const char* key, Bool value) {
   if (!config || !key)
     return False;
-  rime::Config* c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config* c = reinterpret_cast<Config*>(config->ptr);
   if (!c)
     return False;
-  return Bool(c->SetBool(key, value));
+  return c->SetBool(key, value != False);
 }
 
 RIME_API Bool RimeConfigSetInt(RimeConfig* config, const char* key, int value) {
   if (!config || !key)
     return False;
-  rime::Config* c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config* c = reinterpret_cast<Config*>(config->ptr);
   if (!c)
     return False;
   return Bool(c->SetInt(key, value));
@@ -791,7 +855,7 @@ RIME_API Bool RimeConfigSetInt(RimeConfig* config, const char* key, int value) {
 RIME_API Bool RimeConfigSetDouble(RimeConfig* config, const char* key, double value) {
   if (!config || !key)
     return False;
-  rime::Config* c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config* c = reinterpret_cast<Config*>(config->ptr);
   if (!c)
     return False;
   return Bool(c->SetDouble(key, value));
@@ -800,7 +864,7 @@ RIME_API Bool RimeConfigSetDouble(RimeConfig* config, const char* key, double va
 RIME_API Bool RimeConfigSetString(RimeConfig* config, const char* key, const char* value) {
   if (!config || !key || !value)
     return False;
-  rime::Config* c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config* c = reinterpret_cast<Config*>(config->ptr);
   if (!c)
     return False;
   return Bool(c->SetString(key, value));
@@ -809,7 +873,7 @@ RIME_API Bool RimeConfigSetString(RimeConfig* config, const char* key, const cha
 RIME_API Bool RimeConfigClear(RimeConfig* config, const char* key) {
   if (!config || !key)
     return False;
-  rime::Config* c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config* c = reinterpret_cast<Config*>(config->ptr);
   if (!c)
     return False;
   return Bool(c->SetItem(key, nullptr));
@@ -818,67 +882,95 @@ RIME_API Bool RimeConfigClear(RimeConfig* config, const char* key) {
 RIME_API Bool RimeConfigCreateList(RimeConfig* config, const char* key) {
   if (!config || !key)
     return False;
-  rime::Config* c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config* c = reinterpret_cast<Config*>(config->ptr);
   if (!c)
     return False;
-  return Bool(c->SetItem(key, rime::New<rime::ConfigList>()));
+  return Bool(c->SetItem(key, New<ConfigList>()));
 }
 
 RIME_API Bool RimeConfigCreateMap(RimeConfig* config, const char* key) {
   if (!config || !key)
     return False;
-  rime::Config* c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config* c = reinterpret_cast<Config*>(config->ptr);
   if (!c)
     return False;
-  return Bool(c->SetItem(key, rime::New<rime::ConfigMap>()));
+  return Bool(c->SetItem(key, New<ConfigMap>()));
 }
 
 RIME_API size_t RimeConfigListSize(RimeConfig* config, const char* key) {
   if (!config || !key)
     return 0;
-  rime::Config* c = reinterpret_cast<rime::Config*>(config->ptr);
+  Config* c = reinterpret_cast<Config*>(config->ptr);
   if (!c)
     return 0;
-  if (rime::ConfigListPtr list = c->GetList(key)) {
+  if (an<ConfigList> list = c->GetList(key)) {
     return list->size();
   }
   return 0;
 }
 
 const char* RimeGetInput(RimeSessionId session_id) {
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session)
     return NULL;
-  rime::Context *ctx = session->context();
+  Context *ctx = session->context();
   if (!ctx)
     return NULL;
   return ctx->input().c_str();
 }
 
 size_t RimeGetCaretPos(RimeSessionId session_id) {
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session)
     return 0;
-  rime::Context *ctx = session->context();
+  Context *ctx = session->context();
   if (!ctx)
     return 0;
   return ctx->caret_pos();
 }
 
 Bool RimeSelectCandidate(RimeSessionId session_id, size_t index) {
-  rime::shared_ptr<rime::Session> session(rime::Service::instance().GetSession(session_id));
+  an<Session> session(Service::instance().GetSession(session_id));
   if (!session)
     return False;
-  rime::Context *ctx = session->context();
+  Context *ctx = session->context();
   if (!ctx)
     return False;
   return Bool(ctx->Select(index));
 }
 
+Bool RimeSelectCandidateOnCurrentPage(RimeSessionId session_id, size_t index) {
+  an<Session> session(Service::instance().GetSession(session_id));
+  if (!session)
+    return False;
+  Context *ctx = session->context();
+  if (!ctx || !ctx->HasMenu())
+    return False;
+  Schema *schema = session->schema();
+  if (!schema)
+    return False;
+  size_t page_size = (size_t)schema->page_size();
+  if (index >= page_size)
+    return False;
+  const auto& seg(ctx->composition().back());
+  size_t page_start = seg.selected_index / page_size * page_size;
+  return Bool(ctx->Select(page_start + index));
+}
+
 const char* RimeGetVersion() {
   return RIME_VERSION;
 }
 
+void RimeSetCaretPos(RimeSessionId session_id, size_t caret_pos) {
+  an<Session> session(Service::instance().GetSession(session_id));
+  if (!session)
+    return;
+  Context *ctx = session->context();
+  if (!ctx)
+    return;
+  return ctx->set_caret_pos(caret_pos);
+}
+
 RIME_API RimeApi* rime_get_api() {
   static RimeApi s_api = {0};
   if (!s_api.data_size) {
@@ -920,6 +1012,7 @@ RIME_API RimeApi* rime_get_api() {
     s_api.select_schema = &RimeSelectSchema;
     s_api.schema_open = &RimeSchemaOpen;
     s_api.config_open = &RimeConfigOpen;
+    s_api.user_config_open = &RimeUserConfigOpen;
     s_api.config_close = &RimeConfigClose;
     s_api.config_get_bool = &RimeConfigGetBool;
     s_api.config_get_int = &RimeConfigGetInt;
@@ -956,6 +1049,11 @@ RIME_API RimeApi* rime_get_api() {
     s_api.get_caret_pos = &RimeGetCaretPos;
     s_api.select_candidate = &RimeSelectCandidate;
     s_api.get_version = &RimeGetVersion;
+    s_api.set_caret_pos = &RimeSetCaretPos;
+    s_api.select_candidate_on_current_page = &RimeSelectCandidateOnCurrentPage;
+    s_api.candidate_list_begin = &RimeCandidateListBegin;
+    s_api.candidate_list_next = &RimeCandidateListNext;
+    s_api.candidate_list_end = &RimeCandidateListEnd;
   }
   return &s_api;
 }
